%% PDE SOLUTION USING OOFEM

function oofemSolve(PlaneMode,EModulus,Nu,Thickness,NodeSupportX,NodeSupportY,NodeDisplcContrlX,NodeDisplcContrlY,NodeForceX,NodeForceY,Fx,Fy,ux,uy,handles);

% OOFEM INPUT AND RUN
% assemble input file for oofem
clc;
set(handles.listbox1,'String', ['generating input file...';get(handles.listbox1,'String')]);
tic;
load(handles.MeshInput,'p','e','t');
fid = fopen('PdeOofemSolve.in','w');
fprintf(fid,'PdeOofemSolve.out\nResponse of copper matrix.\n');
fprintf(fid,'LinearStatic nsteps 1 renumber 1 nmodules 1\n');
% enable iterative solver instead of direct one, needed IML installed in oofem
% fprintf(fid,'LinearStatic nsteps 1 nmodules 1 lstype 1 smtype 4 stype 0 lstol 1e-5 lsiter 5000 lsprecond 4\n');
fprintf(fid,'vtkxml tstep_all domain_all primvars 1 1 vars 2 4 1 stype 1\n');
if PlaneMode == 1
    fprintf(fid,'domain 2dplanestress\n');
elseif PlaneMode == 2
    fprintf(fid,'domain planestrain\n');
end
fprintf(fid,'OutputManager tstep_all dofman_all element_all\n');
fprintf(fid,'ndofman %d nelem %d ncrosssect 1 nmat 1 nbc 5 nic 0 nltf 1\n',size(p,2),size(t,2));
% coordinates, BC
for i = 1:size(p,2)
    % BC search
    i_D_x = find(i == NodeSupportX); % x support
    i_D_y = find(i == NodeSupportY); % y support
    i_D_r_x = find(i == NodeDisplcContrlX); % x displacement control
    i_D_r_y = find(i == NodeDisplcContrlY); % y displacement control
    i_N_x = find(i == NodeForceX); % x forces
    i_N_y = find(i == NodeForceY); % y forces
    % coordinates input
    fprintf(fid,'node %d coords 2 %g %g',i,p(1,i),p(2,i));
    % Dirichlet BC input - node support
    if or(~isempty(i_D_x),~isempty(i_D_y))
        % x direction
        if ~isempty(i_D_x)
            fprintf(fid,' bc 2 1');
        else
            fprintf(fid,' bc 2 0');
        end
        % y direction
        if ~isempty(i_D_y)
            fprintf(fid,' 1');
        else
            fprintf(fid,' 0');
        end
    end
    % Dirichlet BC input - displacement control
    if or(~isempty(i_D_r_x),~isempty(i_D_r_y))
        % x direction
        if ~isempty(i_D_r_x)
            fprintf(fid,' bc 2 2');
        else
            fprintf(fid,' bc 2 0');
        end
        % y direction
        if ~isempty(i_D_r_y)
            fprintf(fid,' 3');
        else
            fprintf(fid,' 0');
        end
    end
    % node forces
    if or(~isempty(i_N_x),~isempty(i_N_y))
        % x direction
        if ~isempty(i_N_x)
            fprintf(fid,' load 1 4');
        end
        % y direction
        if ~isempty(i_N_y)
            fprintf(fid,' load 1 5');
        end
    end
    % new line
    fprintf(fid,'\n');
    
end
% elements
for i = 1:size(t,2)
    % triangles input
    if PlaneMode == 1
        fprintf(fid,'TrPlaneStress2D %d nodes 3 %d %d %d mat 1 crosssect 1\n',i,t(1,i),t(2,i),t(3,i));
    elseif PlaneMode == 2
        fprintf(fid,'TrPlaneStrain %d nodes 3 %d %d %d mat 1 crosssect 1\n',i,t(1,i),t(2,i),t(3,i));
    end
end
fprintf(fid,'SimpleCS 1 thick %g\n',Thickness);
fprintf(fid,'isole 1 d 0.0 e %g n %g tAlpha 0.0\n',EModulus,Nu);
fprintf(fid,'BoundaryCondition 1 loadTimeFunction 1 prescribedvalue 0.0\n');
fprintf(fid,'BoundaryCondition 2 loadTimeFunction 1 prescribedvalue %g\n',ux);
fprintf(fid,'BoundaryCondition 3 loadTimeFunction 1 prescribedvalue %g\n',uy);
fprintf(fid,'NodalLoad 4 loadTimeFunction 1 Components 2 %g 0.0\n',Fx);
fprintf(fid,'NodalLoad 5 loadTimeFunction 1 Components 2 0.0 %g\n',Fy);
fprintf(fid,'ConstantFunction 1 f(t) 1.0\n');
fclose('all');
set(handles.listbox1,'String', [['Time consumed ',num2str(toc),' sec'];'Done!';get(handles.listbox1,'String')]);
% run oofem, results saved to solve.vtu (examine in ParaView)
set(handles.listbox1,'String', ['OOFEM call...';get(handles.listbox1,'String')]);
tic;
dos('oofem.exe -f PdeOofemSolve.in');
set(handles.listbox1,'String', [['Time consumed ',num2str(toc),' sec'];'Done!';get(handles.listbox1,'String')]);

end
