%% RUN MESH
% creates mesh from recognized contours, saves mesh into mesh.mat
clear all;clc;


%% INPUT PARAMETERS
% maximal lenght of mesh triangle edge
HMax = 0.5;

% maximal number of jiggle mesh iterations
JiggleIter = 10;

% numbering: 0 - nothing, 1 - edge points, 2 - all points, 3 - triangles, 4 - points & triangles
Numbering = 1;


%% BUILDING MESH
fprintf('=== CREATING MESH ===\n');
tic;
load('ContourSolve','Database','BB','HH','PixelSize');
fprintf('creating geometric data...\n');
% creating Decomposed Geometry matrix
MaxLength = 0;
for i = 1:length(Database)
    LengthSolid = size(Database(i).XY(1,1:int16(HMax/PixelSize):end-1),2);
    if LengthSolid >= MaxLength
        MaxLength = LengthSolid;
    end
end
gd = zeros(MaxLength,length(Database));
gd(1:10,1) = [3;4;2*PixelSize;(BB+1)*PixelSize;(BB+1)*PixelSize;2*PixelSize;2*PixelSize;2*PixelSize;(HH+1)*PixelSize;(HH+1)*PixelSize];
sf = 'R1';
ns = [82;49;0];
% for cyklus pres pouzite polygony
for i = 1:length(Database);
    % geometry description matrix
    SolidData = [2;size(Database(i).XY(1,1:int16(HMax/PixelSize):end-1),2);Database(i).XY(1,1:int16(HMax/PixelSize):end-1)';Database(i).XY(2,1:int16(HMax/PixelSize):end-1)'];
    gd(1:length(SolidData),i+1) = SolidData;
    % set formula sf
    sf = [sf,'-P',num2str(i)];
    % name space ns
    if i < 10
        ns(:,i+1) = [80;double(num2str(i));0];
    else
        ns(:,i+1) = [80;double(num2str(i))'];
    end
end
% decomposed geometry
fprintf('decomposing geometry...\n');
dl = decsg(gd,sf,ns);
% mesh initialization
fprintf('generating mesh...\n');
[p,e,t] = initmesh(dl,'Hmax',HMax,'JiggleIter',JiggleIter);
% save mesh to mesh.mat
fprintf('Saving mesh data to mesh.mat...\n');
save('mesh','p','e','t');
% plot mesh
figure(101),hold all,axis equal;
title('Mesh'),xlabel('X'),ylabel('Y');
pdemesh(p,e,t);
switch Numbering
    case 1
        % label edge points
        for i = 1:size(e,2)
            text(p(1,e(1,i)),p(2,e(1,i)),num2str(e(1,i)),'color',[0.3 0.7 0.3],'FontSize',8,'horizontalAlignment','left','verticalAlignment','bottom');
        end
    case 2
        % label all points
        for i = 1:length(p)
            text(p(1,i),p(2,i),num2str(i),'color',[0.3 0.7 0.3],'FontSize',8,'horizontalAlignment','left','verticalAlignment','bottom');
        end
    case 3
        % triangle labels
        for i = 1:length(t)
            x = ([p(1,t(1,i)),p(2,t(1,i))]+[p(1,t(2,i)),p(2,t(2,i))]+[p(1,t(3,i)),p(2,t(3,i))])/3;
            text(x(1),x(2),num2str(i),'color','black','FontSize',8,'horizontalAlignment','center','verticalAlignment','middle');
        end
    case 4
        % label all points
        for i = 1:length(p)
            text(p(1,i),p(2,i),num2str(i),'color',[0.3 0.7 0.3],'FontSize',8,'horizontalAlignment','left','verticalAlignment','bottom');
        end
        % triangle labels
        for i = 1:length(t)
            x = ([p(1,t(1,i)),p(2,t(1,i))]+[p(1,t(2,i)),p(2,t(2,i))]+[p(1,t(3,i)),p(2,t(3,i))])/3;
            text(x(1),x(2),num2str(i),'color','black','FontSize',8,'horizontalAlignment','center','verticalAlignment','middle');
        end
end
fprintf('Done!\n');
fprintf('Time consumed %.1f sec\n',toc);
