%% RUN SOLVE
% loads mesh.mat data and computes 2D deformation, saves results in
% PdeMatlabSolve.txt or PdeOofemSolve.txt/vtu.
clear all;clc;


%% INPUT PARAMETERS
% PdeMode: 0 - Matlab, 1 - OOFEM (oofem.exe file needed)
PdeMode = 0;
% for results from oofem install ParaVieW (vtkxml output format)

% PlaneMode: 0 - plane stress, 1 - plane strain
PlaneMode = 1;

% Young modulus
EModulus = 21358;
% Poisson ratio
Nu = 0.235;
% plate thickness
Thickness = 0.8;

% Dirichlet boundary conditions - node support: ux = uy = 0
% node numbers supported in x direction
NodeSupportX = [1313 1312 701 702 52 716 1309:1311 90 717 1301:1308];
% node numbers supported in y direction
NodeSupportY = [1313 1312 701 702 706 1287 708 1288:1290 710 1291:1294 712 1295 1296 697 1252:1265 698 699 1266:1273];

% Dirichlet boundary conditions - displacement control: determine also ux, uy
% node numbers controlled by x displacement
NodeDisplcContrlX = [153 1274:1282 104 1283:1286];
% node numbers controlled by y displacement
NodeDisplcContrlY = [];

% displacement controll values
ux = 5;
uy = 0;

% node forces
% node numbers loaded in x direction
NodeForceX = [];
% node numbers loaded in y direction
NodeForceY = [];

% force components
Fx = 0;
Fy = 0;


%% SOLUTION OF PDE
if PdeMode == 0
    [u,defStr,p,e,t] = matlabSolve(PlaneMode,EModulus,Nu,Thickness,NodeSupportX,NodeSupportY,NodeDisplcContrlX,NodeDisplcContrlY,NodeForceX,NodeForceY,Fx,Fy,ux,uy);
    red;
elseif PdeMode == 1
    oofemSolve(PlaneMode,EModulus,Nu,Thickness,NodeSupportX,NodeSupportY,NodeDisplcContrlX,NodeDisplcContrlY,NodeForceX,NodeForceY,Fx,Fy,ux,uy);
end
