% function for finding minimal distance between two contours

function r = minimalDistance(xy1,xy2,MaxDist,nParts,DistanceMethod)

if DistanceMethod == 0 || nParts >= min(size(xy1,2),size(xy2,2))
    r = minDist(xy1,xy2);
elseif DistanceMethod == 1
    % segmentation of curves
    if nParts < 10
        nParts = 10;
    end
    LengthPart = max(1,min(int16(length(xy1)/nParts),int16(length(xy2)/nParts)));
    nParts1 = int16(length(xy1)/LengthPart);
    nParts2 = int16(length(xy2)/LengthPart);
    [Distance,j1,j2] = minDist(xy1(:,1:LengthPart:end),xy2(:,1:LengthPart:end));
    % searching minimal distance
    if Distance > 1.5*MaxDist
        r = 2*MaxDist;
    else
        iter = 1;
        res = 1;
        Distance1 = Distance;
        while res > 1e-6*MaxDist
            if iter == 1
                % construct closed curve interval
                Tempxy1 = [xy1,xy1,xy1];
                Tempxy2 = [xy2,xy2,xy2];
                xy1 = Tempxy1(:,(nParts1+j1-int16(nParts/4))*LengthPart:(nParts1+j1+int16(nParts/4))*LengthPart);
                xy2 = Tempxy2(:,(nParts2+j2-int16(nParts/4))*LengthPart:(nParts2+j2+int16(nParts/4))*LengthPart);
            else
                % construct open curve interval
                xy1 = xy1(:,max(1,(j1-int16(nParts/4))*LengthPart):min(length(xy1),(j1+int16(nParts/4))*LengthPart));
                xy2 = xy2(:,max(1,(j2-int16(nParts/4))*LengthPart):min(length(xy2),(j2+int16(nParts/4))*LengthPart));
            end
            % new lengths of curve parts
            if min(int16(length(xy1)/nParts),int16(length(xy2)/nParts)) < 5 || iter == 5
                LengthPart = 1;
            else
                LengthPart = max(1,min(int16(length(xy1)/nParts),int16(length(xy2)/nParts)));
            end
            % distances of segmented curves
            [Distance,j1,j2] = minDist(xy1(:,1:LengthPart:end),xy2(:,1:LengthPart:end));
            % distance residuum
            res = abs(Distance-Distance1);
            Distance1 = Distance;
            iter = iter + 1;
            if iter > 100 || isempty(Distance)
                error('Can not converge to closest point!');
            end
            r = Distance;
        end
    end
end

end


function [r,j1,j2] = minDist(xy1,xy2)

% j1 position number of closest point on curve no 1
% j2 position number of closest point on curve no 2
n1 = size(xy1,2);
n2 = size(xy2,2);
if n1<n2
    R = zeros(2,n1);
    for i = 1:n1
        X1 = xy2(1,:)-xy1(1,i);
        X2 = xy2(2,:)-xy1(2,i);
        X = (X1.^2+X2.^2).^0.5;
        [a,k] = min(X);
        R(1,i) = a;
        R(2,i) = k;
    end
    [r,j1] = min(R(1,:));
    j2 = R(2,j1);
else
    R = zeros(2,n2);
    for i = 1:n2
        X1 = xy1(1,:)-xy2(1,i);
        X2 = xy1(2,:)-xy2(2,i);
        X = (X1.^2+X2.^2).^0.5;
        [a,k] = min(X);
        R(1,i) = a;
        R(2,i) = k;
    end
    [r,j2] = min(R(1,:));
    j1 = R(2,j2);
end

end
